@@ -371,7 +371,9 @@ MCAST_FLOOD_UNREGISTERED = 'mcast_flood_unregistered'
 
 EXTERNAL_PORT_TYPES = (portbindings.VNIC_DIRECT,
                        portbindings.VNIC_DIRECT_PHYSICAL,
-                       portbindings.VNIC_MACVTAP)
+                       portbindings.VNIC_MACVTAP,
+                       portbindings.VNIC_BAREMETAL,
+                       )
 
 NEUTRON_AVAILABILITY_ZONES = 'neutron-availability-zones'
 OVN_CMS_OPTIONS = 'ovn-cms-options'
@@ -394,4 +396,5 @@ OVN_SUPPORTED_VNIC_TYPES = [portbindings.VNIC_NORMAL,
                             portbindings.VNIC_MACVTAP,
                             portbindings.VNIC_VHOST_VDPA,
                             portbindings.VNIC_REMOTE_MANAGED,
+                            portbindings.VNIC_BAREMETAL,
                             ]
