@@ -44,6 +44,7 @@ from ovsdbapp.backend.ovs_idl import idlutils
 from neutron._i18n import _
 from neutron.common.ovn import acl as ovn_acl
 from neutron.common.ovn import constants as ovn_const
+from neutron.common.ovn import exceptions as ovn_exceptions
 from neutron.common.ovn import extensions as ovn_extensions
 from neutron.common.ovn import utils as ovn_utils
 from neutron.common import utils
@@ -756,6 +757,32 @@ class OVNMechanismDriver(api.MechanismDriver):
                     'device_owner': original_port['device_owner']})
             raise OVNPortUpdateError(resource='port', msg=msg)
 
+    def _ovn_update_port(self, plugin_context, port, original_port,
+                         retry_on_revision_mismatch):
+        try:
+            self._ovn_client.update_port(plugin_context, port,
+                                         port_object=original_port)
+        except ovn_exceptions.RevisionConflict:
+            if retry_on_revision_mismatch:
+                # NOTE(slaweq): I know this is terrible hack but there is no
+                # other way to workaround possible race between port update
+                # event from the OVN (port down on the src node) and API
+                # request from nova-compute to activate binding of the port on
+                # the dest node.
+                original_port_migrating_to = original_port.get(
+                    portbindings.PROFILE, {}).get('migrating_to')
+                port_host_id = port.get(portbindings.HOST_ID)
+                if (original_port_migrating_to is not None and
+                        original_port_migrating_to == port_host_id):
+                    LOG.debug("Revision number of the port %s has changed "
+                              "probably during live migration. Retrying "
+                              "update port in OVN.", port)
+                    db_port = self._plugin.get_port(plugin_context,
+                                                    port['id'])
+                    port['revision_number'] = db_port['revision_number']
+                    self._ovn_update_port(plugin_context, port, original_port,
+                                          retry_on_revision_mismatch=False)
+
     def create_port_postcommit(self, context):
         """Create a port.
 
@@ -847,8 +874,8 @@ class OVNMechanismDriver(api.MechanismDriver):
             # will fail that OVN has port with bigger revision.
             return
 
-        self._ovn_client.update_port(context._plugin_context, port,
-                                     port_object=original_port)
+        self._ovn_update_port(context._plugin_context, port, original_port,
+                              retry_on_revision_mismatch=True)
         self._notify_dhcp_updated(port['id'])
 
     def delete_port_postcommit(self, context):
